@@ -25,6 +25,14 @@ import textwrap
 from ._compat import IS_PY38_OR_GREATER
 
 
+# Avoid DeprecationWarnings under Python 3.12 and up
+if IS_PY38_OR_GREATER:
+    astStr = ast.Constant
+    astNum = ast.Constant
+else:  # pragma: no cover
+    astStr = ast.Str
+    astNum = ast.Num
+
 # For AugAssign the operator must be converted to a string.
 IOPERATOR_TO_STR = {
     ast.Add: '+=',
@@ -63,6 +71,54 @@ FORBIDDEN_FUNC_NAMES = frozenset([
     'breakpoint',
 ])
 
+# Attributes documented in the `inspect` module, but defined on the listed
+# objects. See also https://docs.python.org/3/library/inspect.html
+INSPECT_ATTRIBUTES = frozenset([
+    # on traceback objects:
+    "tb_frame",
+    # "tb_lasti",  # int
+    # "tb_lineno",  # int
+    "tb_next",
+    # on frame objects:
+    "f_back",
+    "f_builtins",
+    "f_code",
+    "f_globals",
+    # "f_lasti",  # int
+    # "f_lineno",  # int
+    "f_locals",
+    "f_trace",
+    # on code objects:
+    # "co_argcount",  # int
+    "co_code",
+    # "co_cellvars",  # tuple of str
+    # "co_consts",   # tuple of str
+    # "co_filename",  # str
+    # "co_firstlineno",  # int
+    # "co_flags",  # int
+    # "co_lnotab",  # mapping between ints and indices
+    # "co_freevars",  # tuple of strings
+    # "co_posonlyargcount",  # int
+    # "co_kwonlyargcount",  # int
+    # "co_name",  # str
+    # "co_qualname",  # str
+    # "co_names",  # str
+    # "co_nlocals",  # int
+    # "co_stacksize",  # int
+    # "co_varnames",  # tuple of str
+    # on generator objects:
+    "gi_frame",
+    # "gi_running",  # bool
+    "gi_code",
+    "gi_yieldfrom",
+    # on coroutine objects:
+    "cr_await",
+    "cr_frame",
+    # "cr_running",  # bool
+    "cr_code",
+    "cr_origin",
+])
+
 
 # When new ast nodes are generated they have no 'lineno', 'end_lineno',
 # 'col_offset' and 'end_col_offset'. This function copies these fields from the
@@ -224,7 +280,7 @@ class RestrictingNodeTransformer(ast.NodeTransformer):
         """
         spec = ast.Dict(keys=[], values=[])
 
-        spec.keys.append(ast.Str('childs'))
+        spec.keys.append(astStr('childs'))
         spec.values.append(ast.Tuple([], ast.Load()))
 
         # starred elements in a sequence do not contribute into the min_len.
@@ -244,12 +300,12 @@ class RestrictingNodeTransformer(ast.NodeTransformer):
 
             elif isinstance(val, ast.Tuple):
                 el = ast.Tuple([], ast.Load())
-                el.elts.append(ast.Num(idx - offset))
+                el.elts.append(astNum(idx - offset))
                 el.elts.append(self.gen_unpack_spec(val))
                 spec.values[0].elts.append(el)
 
-        spec.keys.append(ast.Str('min_len'))
-        spec.values.append(ast.Num(min_len))
+        spec.keys.append(astStr('min_len'))
+        spec.values.append(astNum(min_len))
 
         return spec
 
@@ -537,6 +593,10 @@ class RestrictingNodeTransformer(ast.NodeTransformer):
             """
             return self.node_contents_visit(node)
 
+    def visit_Interactive(self, node):
+        """Allow single mode without restrictions."""
+        return self.node_contents_visit(node)
+
     def visit_List(self, node):
         """Allow list literals without restrictions."""
         return self.node_contents_visit(node)
@@ -712,8 +772,8 @@ class RestrictingNodeTransformer(ast.NodeTransformer):
         return self.node_contents_visit(node)
 
     def visit_MatMult(self, node):
-        """Matrix multiplication (`@`) is currently not allowed."""
-        self.not_allowed(node)
+        """Allow multiplication (`@`)."""
+        return self.node_contents_visit(node)
 
     def visit_BoolOp(self, node):
         """Allow bool operator without restrictions."""
@@ -844,11 +904,18 @@ class RestrictingNodeTransformer(ast.NodeTransformer):
                 '"{name}" is an invalid attribute name because it ends '
                 'with "__roles__".'.format(name=node.attr))
 
+        if node.attr in INSPECT_ATTRIBUTES:
+            self.error(
+                node,
+                f'"{node.attr}" is a restricted name,'
+                ' that is forbidden to access in RestrictedPython.',
+            )
+
         if isinstance(node.ctx, ast.Load):
             node = self.node_contents_visit(node)
             new_node = ast.Call(
                 func=ast.Name('_getattr_', ast.Load()),
-                args=[node.value, ast.Str(node.attr)],
+                args=[node.value, astStr(node.attr)],
                 keywords=[])
 
             copy_locations(new_node, node)
@@ -1052,7 +1119,7 @@ class RestrictingNodeTransformer(ast.NodeTransformer):
                 value=ast.Call(
                     func=ast.Name('_inplacevar_', ast.Load()),
                     args=[
-                        ast.Str(IOPERATOR_TO_STR[type(node.op)]),
+                        astStr(IOPERATOR_TO_STR[type(node.op)]),
                         ast.Name(node.target.id, ast.Load()),
                         node.value
                     ],
